@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( IO::Async::Handle );
 
-our $VERSION = '0.66';
+our $VERSION = '0.68';
 
 use IO::Async::Handle;
 use IO::Async::OS;
@@ -53,10 +53,7 @@ C<IO::Async::Listener> - listen on network sockets for incoming connections
  $listener->listen(
     service  => "echo",
     socktype => 'stream',
-
-    on_resolve_error => sub { print STDERR "Cannot resolve - $_[0]\n"; },
-    on_listen_error  => sub { print STDERR "Cannot listen\n"; },
- );
+ )->get;
 
  $loop->run;
 
@@ -74,10 +71,7 @@ This object can also be used indirectly via an C<IO::Async::Loop>:
     on_stream => sub {
        ...
     },
-
-    on_resolve_error => sub { print STDERR "Cannot resolve - $_[0]\n"; },
-    on_listen_error  => sub { print STDERR "Cannot listen\n"; },
- );
+ )->get;
 
  $loop->run;
 
@@ -109,9 +103,10 @@ having the new socket already configured onto it.
 
 =head2 on_stream $stream
 
-An alternative to C<on_accept>, this an instance of L<IO::Async::Stream> when
-a new client connects. This is provided as a convenience for the common case
-that a Stream object is required as the transport for a Protocol object.
+An alternative to C<on_accept>, this is passed an instance of
+L<IO::Async::Stream> when a new client connects. This is provided as a
+convenience for the common case that a Stream object is required as the
+transport for a Protocol object.
 
 This is now vaguely deprecated in favour of using C<on_accept> with a handle
 constructor or class.
@@ -514,6 +509,38 @@ earlier example:
     ...
  );
 
+=head2 Using A Kernel-Assigned Port Number
+
+Rather than picking a specific port number, is it possible to ask the kernel
+to assign one arbitrarily that is currently free. This can be done by
+requesting port number 0 (which is actually the default if no port number is
+otherwise specified). To determine which port number the kernel actually
+picked, inspect the C<sockport> accessor on the actual socket filehandle.
+
+Either use the L<Future> returned by the C<listen> method:
+
+ $listener->listen(
+    addr => { family => "inet" },
+ )->on_done( sub {
+    my ( $listener ) = @_;
+    my $socket = $listener->read_handle;
+
+    say "Now listening on port ", $socket->sockport;
+ });
+
+Or pass an C<on_listen> continuation:
+
+ $listener->listen(
+    addr => { family => "inet" },
+
+    on_listen => sub {
+       my ( $listener ) = @_;
+       my $socket = $listener->read_handle;
+
+       say "Now listening on port ", $socket->sockport;
+    },
+ );
+
 =head1 AUTHOR
 
 Paul Evans <leonerd@leonerd.org.uk>
